package md3.md3model;

import java.io.*;
import java.util.*;

/**
 * <p>Booch utility class that provides .cfg file IO functions.
 *  
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be)
 */
class CFGIO {
	
//----- this information seems to be hard coded into the Q3 engine -----
	private static final int NROFANIMATIONS=25;
	private static final String[] animationNames =
		{ "Death 1", "Dead 1", "Death 2", "Dead 2", "Death 3", "Dead 3",
			"Gesture", "Shoot", "Hit", "Drop Weapon", "Raise Weapon", "Stand With Weapon", "Stand With Gauntlet",
			"Crouched Walk", "Walk", "Run", "Backpedal", "Swim", "Jump Forward", "Land Forward", "Jump Backward", "Land Backward", "Stand Idle", "Crouched Idle", "Turn In Place" };	
	private static final int[] animationTypes = //order of animation types corresponds to order of names!!
		{ AnimationCfg.BOTH, AnimationCfg.BOTH, AnimationCfg.BOTH, AnimationCfg.BOTH, AnimationCfg.BOTH, AnimationCfg.BOTH,
			AnimationCfg.TORSO, AnimationCfg.TORSO, AnimationCfg.TORSO, AnimationCfg.TORSO, AnimationCfg.TORSO, AnimationCfg.TORSO, AnimationCfg.TORSO,
			AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS, AnimationCfg.LEGS };
//----------------------------------------------------------------------
		
	private CFGIO() {} //cannot instantiate
		
	static void loadAnimationCfg(AnimationCfg newAnimCfg, InputStream in) throws IOException {
		BufferedReader bin=new BufferedReader(new InputStreamReader(in));
		
		int i=0; //last read animation, animation order is hard coded!
		int TORSO_OFFSET=-1;
		int firstTORSOFrame=-1; //first of the TORSO animation frames
		boolean IN_HEADER=true; //are we still in the header of the .cfg file?
		
		//parse file
		String line=bin.readLine();
		while (line!=null) try {
			line.trim();
			
			if (!line.equals("") && !line.startsWith("//")) { //ignore empty lines and comments
				if (IN_HEADER && line.startsWith("sex")) {
					//parse line: sex [m | f | ...]
					StringTokenizer st=new StringTokenizer(line, " \t\n\r\f/");
					
					st.nextToken(); //throw away first token
					String sexStr=st.nextToken();
					if (sexStr.length()!=1)
						throw new IOException("illegal sex value: " + sexStr);
					else
						newAnimCfg.sex=sexStr.charAt(0);
				}
				else if (IN_HEADER && line.startsWith("headoffset")) {
					//parse line: headoffset X Y Z
					StringTokenizer st=new StringTokenizer(line, " \t\n\r\f/");
					
					st.nextToken(); //throw away first token
					
					newAnimCfg.headOffset=
						new md3.util.Vec3(new Float(st.nextToken()).floatValue(),									
					                    new Float(st.nextToken()).floatValue(),
					                    new Float(st.nextToken()).floatValue());
				}
				else if (IN_HEADER && line.startsWith("footsteps")) {
					//parse line: footsteps [normal | mech | ...]
					StringTokenizer st=new StringTokenizer(line, " \t\n\r\f/");
					
					st.nextToken(); //throw away first token
					newAnimCfg.footsteps=st.nextToken().trim();
				}
				else {
					//assume it's an animation line
					IN_HEADER=false; //no longer in header
					
					MD3Animation animation=MD3ModelFactory.getFactory().makeMD3Animation(line);
										
					animation.name=animationNames[i];
					animation.type=animationTypes[i++];				
	
					//workaround for strange numbering in animation.cfg: skip TORSO frames
					//for LEGS animation lines...
					if (animation.type==AnimationCfg.LEGS) {
						//when first LEGS animation is found, calc # of TORSO frames
						if (TORSO_OFFSET==-1)
							TORSO_OFFSET=animation.first-firstTORSOFrame;
												
						animation.first-=TORSO_OFFSET;
						animation.offset=TORSO_OFFSET;
					}
					else if (animation.type==AnimationCfg.TORSO)
						if (firstTORSOFrame==-1)
							firstTORSOFrame=animation.first;

					newAnimCfg.putAnimation(animation);
				}
			}
			
			line=bin.readLine();
		}
		catch (Exception e) {
			throw new IOException(e.getMessage());
		}
	}
	
	static void loadAnimation(MD3Animation newAnimation, String line) throws IOException {
		//parse line:
		//first frame, num frames, looping frames, frames per second				
		try {
			StringTokenizer st=new StringTokenizer(line, " \t\n\r\f/");
			
			newAnimation.first=new Integer(st.nextToken()).intValue();									
			newAnimation.num=new Integer(st.nextToken()).intValue();
			newAnimation.looping=new Integer(st.nextToken()).intValue();
			newAnimation.fps=new Integer(st.nextToken()).intValue();
		} catch (Exception e) {
			throw new IOException("corrupt animation line");
		}
	}
}