package md3.md3model;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

import md3.util.*;

/**
 * <p>Booch utility class that provides Image IO functions. A method
 * is provided that you can use to convert java.awt.Image data into an
 * MD3Texture object.
 *  
 * @see md3.md3model.MD3Texture
 *
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be)
 */
class ImageIO {

	private ImageIO() {} //cannot instantiate

  /**
   * <p>Convert the given java.awt.Image into a MD3Texture object with OpenGL
   * compatible texture data. The texture bytes that end up in the data member
   * of the MD3Texture object have a RGBA structure and are unsigned.
   *
   * @param tex MD3Texture object that should hold the read data.
   * @param sourceImg Source image data.
   */
	static void loadImage(MD3Texture tex, Image sourceImg) throws IOException {
    tex.width=sourceImg.getWidth(null);
    tex.height=sourceImg.getHeight(null);
    
    int[] pix=new int[tex.width * tex.height];
	  PixelGrabber pg=new PixelGrabber(sourceImg, 0, 0, tex.width, tex.height, pix, 0, tex.width);
		try {
		  pg.grabPixels();
		} catch (InterruptedException e) {
			throw new IOException("interrupted waiting for pixels");
		}
		
		if ((pg.getStatus() & ImageObserver.ABORT) != 0)
			throw new IOException("image fetch aborted or errored");
			
    tex.data=new byte[pix.length*4];
    int dataPtr=0;
    for (int i=0;i<pix.length;i++) {
    	//pixels in pix are in ARGB order, convert to RGBA
      tex.data[dataPtr++]=ByteUtils.getByte(pix[i], 1);
      tex.data[dataPtr++]=ByteUtils.getByte(pix[i], 2);
      tex.data[dataPtr++]=ByteUtils.getByte(pix[i], 3);
      tex.data[dataPtr++]=ByteUtils.getByte(pix[i], 0);
    }
	}
}