package md3.md3model;

import java.io.*;

import md3.util.*;

/**
 * <p>The MD3Tag class represents a so called 'tag', as they are found
 * in the MD3 model files. Tags can be seen as references to other
 * attached models, i.e. the weapon a player holds.
 *
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be)
 */
public class MD3Tag implements java.io.Serializable {
	
  /**
   * <p>Name of 'tag' as it's usually called in the md3 files. Try
   * to see it as a sub-mesh/seperate mesh-part.
   */
  public String name;
    
  /**
   * <p>Position of tag relative to the model that contains the tag.
   */
  public Vec3 position=new Vec3(); 
  
  /**
   * <p>3x3 rotation matrix.
   */
  public float[][] matrix=new float[3][3];
  
  /**
   * <p>Constructor that creates a new tag object and initializes
   * it with the data coming from the input stream.
   */
  protected MD3Tag(DataInput din) throws IOException {
  	MD3IO.loadTag(this, din);
  }
  
  /**
   * <p>Constructor that creates a new empty tag object.
   */
  protected MD3Tag() {
  }
}