package md3.md3model;

import java.io.*;
import java.awt.*;

/**
 * <p>Objets of this class represent textures with associated metadata. The
 * texture data is stored in the OpenGL compatible RGBA format.
 *  
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be)
 */
public class MD3Texture implements Serializable {
	
	/**
	 * <p>Name as specified in the originating .md3 model or .skin file.
	 */
	public String name;
	
  /**
   * <p>Filename of actual file from which data was loaded.
   */
	public String loadFilename;

  /**
   * <p>Actual texture bitmap data. These are unsigned bytes in RGBA order.
   */
  public byte[] data;
  public int width, height; // size of texture data
  	
  /**
   * <p>Load the texture data from the given input stream and put it in
   * the data array.
   *
   * <p>TGA or JPG input is selected depending on the loadFilename, so
   * make sure it is set before calling this method!
   */
  protected void getTextureData(InputStream in) throws IOException {
  	if (loadFilename.toLowerCase().endsWith(".jpg")) {
			//read image data into byte array
			ByteArrayOutputStream bout=new ByteArrayOutputStream(in.available());  		
	    byte[] buf=new byte[1024];
	    int len=0;
	    while((len=in.read(buf)) > 0 )
	    	bout.write(buf, 0, len);  		
			byte[] imgData=bout.toByteArray();
	
	    // load texture bitmap data
      MediaTracker tracker=new MediaTracker(new Canvas()); //use dummy component			Image img=Toolkit.getDefaultToolkit().createImage(imgData);
      Image img=Toolkit.getDefaultToolkit().createImage(imgData);
      tracker.addImage(img, 1);
      try { tracker.waitForID(1); } catch (InterruptedException e) {}
	
	 		ImageIO.loadImage(this, img);
	  }
	  else if (loadFilename.toLowerCase().endsWith(".tga"))
     	TGAIO.loadTGA(this, in);
  }

  /**
   * <p>Constructor for use in subclasses that loads texture data from the
   * specified file and initializes a new MD3Texture object with the read data/metadata.
   *
   * @param name Name of the texture as specified in .md3 or .skin file.
   * @param loadFilename Full name of file from which texture data will be loaded.
   */
	protected MD3Texture(String name, String loadFilename) throws IOException {
		loadFilename=loadFilename.trim();
		FileInputStream fin=new FileInputStream(loadFilename);
    this.name=name.trim();
    this.loadFilename=loadFilename;
		this.getTextureData(fin);
    fin.close();
	}
	
  /**
   * <p>Constructor for use in subclasses that loads texture data from the sepcified
   * input stream and initializes a new MD3Texture object with the read data/metadata.
   * The given loadFilename is stored in the corresponding data member.
   *
   * @param name Name of the texture as specified in .md3 or .skin file.
   * @param loadFilename Full name of file from which texture data is (supposedly) loaded.
   * @param in Stream from to load texture data from.
   */
	protected MD3Texture(String name, String loadFilename, InputStream in) throws IOException {
		this.name=name.trim();
		this.loadFilename=loadFilename.trim();
		this.getTextureData(in);
	}
	
	/**
	 * <p>Create a new uninitialized MD3Texture object.
	 */
	protected MD3Texture() {	
	}	
}