package md3.md3view;

import md3.md3model.*;

/**
 * <p>This class extends MD3Model objects with animation state data.
 *  
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be)
 */
public class MD3GLModel extends MD3Model {
	
  /**
   * <p>Currently rendered animation key frame of this model. Note that this
   * model could actually be only a part of a larger model. Every part
   * has its own current frame!
   */
  public int currentFrame;
  
  /**
   * <p>Next animation key frame of the model.
   */
  public int nextFrame;
  
  /**
   * <p>Interpolation position between currentFrame and nextFrame. The value
   * should be in the interval [0,1]!
   */
  public transient float interpolationFraction;
  
  protected MD3GLModel(String filename) throws java.io.IOException {
  	super(filename);
  	this.currentFrame=0;
  	this.nextFrame=this.boneFrameNum>0?1:0;
  	this.interpolationFraction=0.0f;
  }
  
  protected MD3GLModel(String name, java.io.InputStream in) throws java.io.IOException {
  	super(name, in);
  	this.currentFrame=0;
  	this.nextFrame=this.boneFrameNum>0?1:0;
  	this.interpolationFraction=0.0f;  	
  }
  
  public void accept(MD3GLModelVisitor v) {    
  	v.visit(this);
  }
}