package md3.md3view;

import java.util.*;

import md3.md3model.*;

/**
 * <p>An MD3Model visitor that walks through a MD3Model structure and
 * replaces all textures the structure uses with a specified texture.
 *
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be) 
 */
public class MD3ModelApplyTextureVisitor extends MD3ModelVisitor {
  private MD3Texture texture;
  
  /** 
   * <p>Create a new visitor.
   *
   * @param texture The texture to apply to the models.
   */
  public MD3ModelApplyTextureVisitor(MD3Texture texture) {
    this.texture=texture;
  }
  
  public void visit(MD3Model model) {
    //apply texture to this model
    for (int i=0;i<model.meshNum;i++)
      for (int j=0;j<model.meshes[i].textureNum;j++)
        model.meshes[i].textures[j]=texture;        
        
    //apply texture to linked models
    Iterator it=model.linkedModels();
    while (it.hasNext())
      ((MD3Model)it.next()).accept(this);
  }
}