package md3.md3view;

import java.io.*;
import java.util.*;

import md3.md3model.*;
import md3.util.*;

/**
 * <p>An MD3Model visitor that walks through a MD3Model structure and
 * saves all textures of the visited models to files in a given directory.
 *
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be) 
 */
public class MD3ModelExportTexturesVisitor extends MD3ModelVisitor {
	
	private String targetDir;
	
	public MD3ModelExportTexturesVisitor(String targetDir) {
		this.targetDir=targetDir;
	}
				
	public void visit(MD3Model model) {
		for (int i=0;i<model.meshNum;i++) {
			MD3Mesh mesh=model.meshes[i];
			for (int j=0;j<mesh.textureNum;j++)
				if (mesh.textures[j]!=null) try {
					File f=new File(targetDir, new File(mesh.textures[j].loadFilename).getName());
					if (!f.exists()) {
	          FileOutputStream out=new FileOutputStream(f);
	          InputStream in=MD3View.instance().getInputStreamForPath(mesh.textures[j].loadFilename);	
	          
	          ByteUtils.copyAllBytes(in, out);
					
					  in.close();
	          out.close();						
					}
				} catch (IOException e) {
					MD3View.instance().showExceptionDialog(e.getMessage());
				}
				
		}
		
    Iterator it=model.linkedModels();
    while (it.hasNext())
      ((MD3Model)it.next()).accept(this);		
	}	
}