package widgets.awt;

import java.awt.*;

/**
 * <p>This class implements a Panel container with a 3D look.
 *
 * @author Erwin Vervaet (erwin@cs.kuleuven.ac.be)
 */
public class Panel3D extends Panel {

  /**
   * <p>Depressed 3D look.
   */
  public static final int DEPRESSED = 1;

  /**
   * <p>Protruded 3D look.
   */
  public static final int PROTRUDED = 2;

  private int type=PROTRUDED;

  /**
   * <p>Default constructor. Initially the 3D type is PROTRUDED.
   */
  public Panel3D() {
  	super();
  }

  /**
   * <p>Create a 3D panel with given layout manager. Initially the
   * 3D type is PROTRUDED.
   */
  public Panel3D(LayoutManager lm) {
  	super(lm);
  }

  /**
   * <p>Set the type of 3D look the panel will have. Either PROTRUDED
   * or DEPRESSED.
   */
  public void setPanel3DType(int t) {
    if (t == DEPRESSED || t == PROTRUDED) {
      type = t;
      repaint();
    }
    else
    	throw new IllegalArgumentException(String.valueOf(t));
  }

  public void paint(Graphics g) {
    super.paint(g);
    
    if(type == DEPRESSED)
      g.setColor(Color.black);
    else
      g.setColor(Color.white);
      
    g.drawLine(0, 0, getWidth()-1, 0);
    g.drawLine(0, 0, 0, getHeight()-1);
    
    if(type == DEPRESSED)
      g.setColor(Color.white);
    else
      g.setColor(Color.black);
      
    g.drawLine(0, getHeight()-1, getWidth()-1, getHeight()-1);
    g.drawLine(getWidth()-1, 0, getWidth()-1, getHeight()-1);
  }
  
  public Insets getInsets() {
  	return new Insets(1, 4, 1, 4);
  }
}